#include "ADC.h"
#include <avr/io.h> 
#include <string.h>
#include <stdint.h>
#include "../hw-layout.h"

// Functions to access the internal ADC of ATmega targets.

// required settings:
// #define F_CPU                20000000
// #define ADC_CHANNELS         4

// internal ADC reference configuration
static uint8_t adcReference=(1<<REFS0); // = VCC 5V

// get a sample from ADC chip.
// chanell must be in range 0-7.
uint16_t getADC(const int channel) {
    ADMUX = adcReference + channel;
    // start one sample
    ADCSRA |= (1<<ADSC);
    // wait for the result
    loop_until_bit_is_clear(ADCSRA, ADSC);
    return ADC;
}

// initialize the ADC 
void initADC() {
    ADCSRA |= (1<<ADEN) | (1<<ADPS2) | (1<<ADPS1) | (1<<ADPS0);
}

// select reference
// On ATmega, the value can be "VCC", "1.1V or "2.56V"
// On ATxmega, the value can be "INT1V", "INTVCC", "AREFA" or "AREFB"
// returns 1 on success
uint8_t setAREF(const char* name) {
    if (strcmp(name,"VCC")==0)
        adcReference=(1<<REFS0);     
    else if (strcmp(name,"1.1V")==0)
        adcReference=(1<<REFS1);
    else if (strcmp(name,"2.56V")==0)
        adcReference=((1<<REFS1) | (1<<REFS0));
    else if (strcmp(name,"EXT")==0)     
        adcReference=0;                        
    else 
        return 0;
    // Perform one sample to initialize the ADC on the new reference
    getADC(0);
    return 1;
}

