#include "ADC.h"
#include <avr/io.h> 
#include <string.h>
#include <stdint.h>
#include "../hw-layout.h"

// Functions to access the internal ADC of Xmega targets.

// required settings:
// #define F_CPU                32000000
// #define ADC_CHANNELS         4

// Note: For all functional registers, I used the old syntax because
// the structures of the ADC are incorrectly defined in WinAVR.

// get a sample from ADC chip.
// chanell must be in range 0-15.
uint16_t getADC(const int channel) {
    ADCA_CH0_MUXCTRL = (channel<<3);    
    // flush the previous result
    ADCA_CTRLA |= 2; // 2=ADC_FLUSH_bm but not defined in WinAVR
    // start one sample
    ADCA_CH0_INTFLAGS=1;
    ADCA_CH0_CTRL |= ADC_CH_START_bm;
    // wait for the result
    while((ADCA_CH0_INTFLAGS & 1)==0);
    return ADCA_CH0RES;
}

// initialize the ADC 
void initADC() {
    ADCA_CTRLA = ADC_ENABLE_bm;
    ADCA_CTRLB = ADC_RESOLUTION_12BIT_gc;
    ADCA_REFCTRL = ADC_REFSEL_INT1V_gc | ADC_BANDGAP_bm;
    ADCA_PRESCALER = ADC_PRESCALER_DIV64_gc; 
    ADCA_CH0_CTRL = ADC_CH_INPUTMODE_SINGLEENDED_gc; 
}

// select reference
// The value can be "INT1V", "INTVCC", "AREFA" or "AREFB"
// returns 1 on success
uint8_t setAREF(const char* name) {
    if (strcmp(name,"INT1V")==0)
        ADCA_REFCTRL = ADC_REFSEL_INT1V_gc | ADC_BANDGAP_bm; 
    else if (strcmp(name,"INTVCC")==0)
        ADCA_REFCTRL = ADC_REFSEL_VCC_gc   | ADC_BANDGAP_bm; 
    else if (strcmp(name,"AREFA")==0)
        ADCA_REFCTRL = ADC_REFSEL_AREFA_gc | ADC_BANDGAP_bm; 
    else if (strcmp(name,"AREFB")==0)     
        ADCA_REFCTRL = ADC_REFSEL_AREFB_gc | ADC_BANDGAP_bm;                        
    else 
        return 0;
    // Perform one sample to initialize the ADC on the new reference
    getADC(0);
    return 1;
}

