#ifndef _CP2200_H__
#define _CP2200_H__

#include "../uip/uip.h"

// Structure for the configuration in flash
struct flashconfig {
    u8_t dhcpenable;
    u8_t ipaddr[4];
    u8_t netmask[4];
    u8_t gateway[4];
    char authentication[31];
    // you can add more settings here, up to 512 bytes in total
};

// Storage for the configuration
extern struct flashconfig config;

// Location of the configuration in flash
#define FLASHCONFIG 0;

// Read from network device
u16_t network_device_read(void);

// Send to network device
void network_device_send(void);

// Reset the ethernet controller
void eth_RESET(void);

// Initialize physical connection
void init_PHY(void);

// Initialize the MAC address
void init_MAC(void);

// Check if the physical link is up 
uint8_t isLinkUp(void);  

// Read IP configuration from flash.
// This is already called during ethernet controller initalisation.
void readFlashConfig(void);
  
// Write configuration into flash memory.
void writeFlashConfig(void);

// Write a byte to the CP2200 chip
void write_CP2200(int adr, char value);

// Read a byte from the CP2200 chip
u8_t read_CP2200(int adr);

// write a single byte into the flash.
// the flash must be erased before that.
void writeflash(u16_t adr, u8_t data);

// Read a String from the flash memory at the given adress.
void readFlashString(char* buffer, u16_t adr);

// Write a String to the flash memory at the given adress.
// The string must be 0-512 characters long and it must start at a 512-byte boundary.
void writeFlashString(char* buffer, u16_t adr);

// Read a single character from the flash memory at the given adress.
u8_t readFlashChar(u16_t adr);

#ifdef EMAIL_APP

  // Read a String from the flash memory at the given adress and encode it into HTML.
  void readFlashStringEncoded(char* buffer, u16_t adr);
  
#endif //EMAIL_APP

#endif // _CP2200_H__