#include <avr/interrupt.h>
#include <avr/io.h>
#include "../uip/clock.h"
#include "../uip/timer.h"

// This file creates a system clock timer which counts up every 1ms.
// It is used by uIP to perform time related stuff.

// I also added some code for the SD/MMC card reader which decrements
// the TimingDelay variable every 100ms until zero.

static volatile int timercounter;

#ifdef CARDSLOT
    static uint8_t mmcTimerPrescaler;
    volatile uint8_t TimingDelay;    // For mmc.c. Counts down every 100ms
#endif
    
#if defined TCCR0
    // for ATmega128
    #define CLOCK_ISR  ISR(TIMER0_COMP_vect)
    
    void clock_init(void)
    {
        cli();
        TCCR0 = 0x0F; // CTC mode and prescaler divide by 1024
        OCR0 = F_CPU/1024000; // frequency 1kHz
        TIMSK = 0x02; // enable output compare interrupt
        sei();
    }

#elif defined TCCR0A
    // For other Atmega
    #define CLOCK_ISR  ISR(TIMER0_COMPA_vect)

    void clock_init(void)
    {
        cli();
        TCCR0A = 0x02;
        TCCR0B = 0x05;
        OCR0A = F_CPU/1024000; 
        TIMSK0 = 0x02;
        sei();
    }

#elif defined TCC0
    // for Xmega
    #define CLOCK_ISR  ISR(TCC0_OVF_vect)

    void clock_init(void)
    {
        cli();
        PMIC.CTRL |= 0x07;
        TCC0.CNT = 0x00;
        TCC0.PER =  31250;
        TCC0.CTRLA = 0x01;
        TCC0.CTRLB = 0x00;
        TCC0.INTCTRLA = 0x03;
        sei();
    }

#else
    #error You need to edit driver/clock.c to initialize a timer counter.
#endif
    
// Interrupt service routine for the 1kHz timer
CLOCK_ISR
{
    timercounter++;

    #ifdef CARDSLOT
    if (++mmcTimerPrescaler>=100) {
        mmcTimerPrescaler=0;
        TimingDelay = (TimingDelay==0) ? 0 : TimingDelay-1;
    }
    #endif
}

// This function is called by uIP to query the system time in seconds.
int clock_time(void)
{
    int tmp;
    cli();
    tmp=timercounter;
    sei();
    return tmp;
}
