#ifndef __SERIALCONSOLE_H_
#define __SERIALCONSOLE_H_

#include <stdint.h>

// Required settings, usually in the Makefile:
// #define SERIAL_BITRATE 115200
// #define F_CPU 20000000

// Select one or more serial ports on ATmega controllers:
// #define USE_SERIAL0
// #define USE_SERIAL1
// #define USE_SERIAL2
// #define USE_SERIAL3

// Select one or more serial ports on Xmega controllers:
// #define USE_USARTC0
// #define USE_USARTD0
// #define USE_USARTE0
// #define USE_USARTF0
// #define USE_USARTC1
// #define USE_USARTD1
// #define USE_USARTE1
// #define USE_USARTF1
//
// Also set all TxD Pins to output mode, example for USARTE0:
// #define init_serial_pins()       {                                     
//                                     PORTE.OUTSET = (1<<3); 
//                                     PORTE.DIRSET = (1<<3); 
//                                   }  

// If you select more than one single serial port, then all ports send the
// same data and receive into the same shared buffer.

// In the software, lines are always terminated with \n.
// If terminal mode is enabled, line breaks are converted automatically:
// - in sending direction, line breaks are sent as \r\n.
// - in receiving direction, line breaks my be terminated by \r, \n or \r\n.
// - backspace characters in receiving direction delete the last received character from the input buffer.
#define TERMINAL_MODE 1

// Enable echo on serial interface
#define SERIAL_ECHO 0

// Size of input buffer in chars, max 254 (output is not buffered)
#define SERIAL_INPUT_BUFFER_SIZE 80

// Initialize the serial port and bind it to stdin and stdout.
// Don't forget to set the TxD pin to output mode in case of ATxmega.
void initserial(void);

// Returns 1 if the serial port has received at least one character
// so that following calls to getc() do not block.
uint8_t receivedChar(void);

// Returns 1 if the serial port has received a line break
// so that following calls to gets() and scanf() do not block.
uint8_t receivedLine(void);

#endif //__SERIALCONSOLE_H_
