#ifndef _HW_LAYOUT_CRUMB644_NET_v1_H__
#define _HW_LAYOUT_CRUMB644_NET_v1_H__

// Crumb644-NET version 1.0 from www.chip45.com

// Macros for access to address/data bus
#define ADBUS_output()       DDRA = 255
#define ADBUS_input()        DDRA = 0 
#define ADBUS_write(value)   PORTA = (value)
#define ADBUS_read()         PINA

// Macros for the control lines of the adress/data bus
#define ALE(value)           writeBit(PORTC,3,value)
#define nWR(value)           writeBit(PORTC,4,value)
#define nRD(value)           writeBit(PORTC,5,value)

// Macros for chip-select signals
#define nCSADC(value)        writeBit(PORTC,2,value)
#define nCSETH(value)        writeBit(PORTC,6,value)
   
// The status LED is accessed in open-drain mode
#define STATUS_LED_on()      { \
                               writeBit(PORTC,0,0); \
                               writeBit(DDRC,0,1); \
                             }
#define STATUS_LED_off()     writeBit(DDRC,0,0)

// Initialisation of the IO pins for ethernet, status LED's and Reset jumper.
#define init_io_pins()       { \
                               DDRA=0; \
                               DDRC=(1<<2)+(1<<3)+(1<<4)+(1<<5)+(1<<6); \
                               PORTC=(1<<2)+(1<<3)+(1<<4)+(1<<5)+(1<<6); \
                             }   

// The serial port(s) to be used for console
#define USE_SERIAL0
                             
// Config reset jumper on ISP connector.
// PB5 is used as input with pull-ups and PB7 is used as output
#define RST_JUMPER_init()           { \
                                       writeBit(DDRB,7,1); \
                                       writeBit(DDRB,5,0); \
                                       writeBit(PORTB,5,1); \
                                    }
#define RST_JUMPER_write(value)     writeBit(PORTB,7,value)
#define RST_JUMPER_read             readBit(PINB,5)
#define RST_JUMPER_exit()           { \
                                      writeBit(DDRB,7,0); \
                                      writeBit(PORTB,5,0); \
                                    }

// Serial ADC communication to MCP3204 chip.
// PB5 is the data line from AVR to ADC
// PB6 is the data line from ADC to AVR
// PB7 is the clock line
#define ADC_CHANNELS         4
#define ADC_PORT             PORTB
#define ADC_DDR              DDRB
#define ADC_CLK(value)       writeBit(PORTB,7,value)
#define ADC_write(value)     writeBit(PORTB,5,value)
#define ADC_read()           readBit(PINB,6)
#define init_adc_pins()      { \
                               writeBit(DDRB,5,1); \
                               writeBit(DDRB,6,0); \
                               writeBit(PORTB,7,0); \
                               writeBit(DDRB,7,1); \
                             }
                             
// Settings for the Email application
#define email_PORT       PIND
                             
// Definition of i/o port bits that are write-protected.
// These pins cannot be accessed via io-commands d, p and o.
// 0x00 means that the whole port is not protected.
#define protect_PORTA        0xFF
#define protect_PORTB        0x00
#define protect_PORTC        0xFC
#define protect_PORTD        0x00

#endif // _HW_LAYOUT_CRUMB644_NET_v1_H__

