#ifndef _HW_LAYOUT_CRUMB644_NET_v2_XL_H__
#define _HW_LAYOUT_CRUMB644_NET_v2_XL_H__

// Crumb644-NET version 2.0 or 2.1 from www.chip45.com
//
// with shift registers 74HC165 additional inputs
// PD2 = Shift pulse (Pin 2 CP)
// PD3 = Data in     (Pin 9 Q7)
// PD4 = Load pulse  (Pin 1 nPL)
//
// with shift registers 74HC595 additional outputs
// PD5 = Store pulse (Pin 12 STcp)
// PD6 = Shift pulse (Pin 11 SHcp)
// PD7 = Data out    (Pin 14 DS)

// Macros for access to address/data bus
#define ADBUS_output()       DDRA = 255
#define ADBUS_input()        DDRA = 0 
#define ADBUS_write(value)   PORTA = (value)
#define ADBUS_read()         PINA

// Control signals to the ethernet chip
#define ALE(value)           writeBit(PORTC,3,value)
#define nWR(value)           writeBit(PORTC,4,value)
#define nRD(value)           writeBit(PORTC,5,value)
#define nCSETH(value)        writeBit(PORTC,6,value)
   
// The status LED is accessed in open-drain mode
#define STATUS_LED_on()      { \
                               writeBit(PORTC,0,0); \
                               writeBit(DDRC,0,1); \
                             }
#define STATUS_LED_off()     writeBit(DDRC,0,0)   
   
// Initialisation of the IO pins for ethernet, status LED's and Reset jumper.
#define init_io_pins()       { \
                               DDRA=0; \
                               DDRC=(1<<3)+(1<<4)+(1<<5)+(1<<6); \
                               PORTC=(1<<3)+(1<<4)+(1<<5)+(1<<6); \
                               DDRD=(1<<2)+(1<<4)+(1<<5)+(1<<6)+(1<<7); \
                               PORTD=(1<<4); \
                             }   
                             
// The serial port(s) to be used for console 
#define USE_SERIAL0                              
                             
// Config reset jumper on ISP connector, PB=output, PB5=input with pull-up
#define RST_JUMPER_init()           { \
                                      writeBit(DDRB,7,1); \
                                      writeBit(DDRB,5,0); \
                                      writeBit(PORTB,5,1); \
                                    }
#define RST_JUMPER_write(value)     writeBit(PORTB,7,value)
#define RST_JUMPER_read             readBit(PINB,5)
#define RST_JUMPER_exit()           { \
                                      writeBit(DDRB,7,0); \
                                      writeBit(PORTB,5,0); \
                                    }

// The ADC inputs are not accessible on this harware.
#define ADC_CHANNELS         0

// Settings for the Email application
#define email_PORT       PIND

// Definition of i/o port bits that are protected.
// These pins cannot be accessed via io-commands d, p and o.
// 0x00 means that the whole port is not protected.
#define protect_PORTA        0xFF
#define protect_PORTB        0x00
#define protect_PORTC        0xFC
#define protect_PORTD        0xFC

// Number of shift register inputs and outputs
#define XL_INPUTS            24
#define XL_OUTPUTS           24

// Macros for additional inputs through shift registers (Port X)
// Some commands are duplicated to slow down the serial interface a little
#define SHIFTin              { \
                               writeBit(PORTD,2,1); \
                               writeBit(PORTD,2,1); \
                               writeBit(PORTD,2,0); \
                               writeBit(PORTD,2,0); \
                             }
#define DATAin               readBit(PIND,3)
#define LOADin               { \
                               writeBit(PORTD,4,0); \
                               writeBit(PORTD,4,0); \
                               writeBit(PORTD,4,1); \
                               writeBit(PORTD,4,1); \
                             } 

// Macros for additional outputs through shift registers (Port Y)
// Some commands are duplicated to slow down the serial interface a little
#define STOREout             { \
                               writeBit(PORTD,5,1); \
                               writeBit(PORTD,5,1); \
                               writeBit(PORTD,5,0); \
                               writeBit(PORTD,5,0); \
                             }
#define SHIFTout             { \
                               writeBit(PORTD,6,1); \
                               writeBit(PORTD,6,1); \
                               writeBit(PORTD,6,0); \
                               writeBit(PORTD,6,0); \
                             }
#define DATAout(value)       writeBit(PORTD,7,value)

#endif // _HW_LAYOUT_CRUMB644_NET_v2_XL_H__

