#ifndef _HW_LAYOUT_CRUMBX1_NET_XL_H__
#define _HW_LAYOUT_CRUMBX1_NET_XL_H__

// CrumbX1-NET version 1.0-1.2 from www.chip45.com
//
// with shift registers 74HC165 additional inputs
// PD0 = Data in     (Pin 9 Q7)
// PD1 = Load pulse  (Pin 1 nPL)
// PD4 = Shift pulse (Pin 2 CP)
//
// with shift registers 74HC595 additional outputs
// PD2 = Data out    (Pin 14 DS)
// PD3 = Store pulse (Pin 12 STcp)
// PD4 = Shift pulse (Pin 11 SHcp)
//
// PD4 ist used to clock all input and output shift registers.
//
// Please note that the shift register extension has not been tested with a CrumbX1 module!
// If you tested it please report the result to stefan@stefanfrings.de.

// Macros for access to address/data bus
#define ADBUS_output()       PORTF.DIR = 255
#define ADBUS_input()        PORTF.DIR = 0 
#define ADBUS_write(value)   PORTF.OUT = (value)
#define ADBUS_read()         PORTF.IN

// swap bit order of the address/data bus
#define SWAP_ADBUS           1

// Control signals to the ethernet chip
#define ALE(value)           writeBit(PORTE.OUT,7,value)
#define nWR(value)           writeBit(PORTE.OUT,5,value)
#define nRD(value)           writeBit(PORTE.OUT,6,value)
#define nCSETH(value)        writeBit(PORTE.OUT,4,value)
#define nRSTETH(value)       writeBit(PORTA.OUT,6,value)
   
// Status LED
#define STATUS_LED_on()      PORTA.OUTCLR=(1<<3)
#define STATUS_LED_off()     PORTA.OUTSET=(1<<3)
      
// Initialisation of the IO pins for ethernet, status LED's, Reset jumper and shift 
#define init_io_pins()       { \
                               PORTA.PIN3CTRL = PORT_OPC_WIREDAND_gc; \
                               PORTA.PIN4CTRL = PORT_OPC_WIREDAND_gc; \
                               PORTA.PIN6CTRL = PORT_OPC_WIREDAND_gc; \
                               PORTA.OUTSET = (1<<3)+(1<<4)+(1<<6); \
                               PORTA.DIRSET = (1<<3)+(1<<4)+(1<<6); \
                               PORTA.DIRSET = (1<<1)+(1<<2)+(1<<3)+(1<<4); \
                               PORTE.OUTSET = (1<<4)+(1<<5)+(1<<6); \
                               PORTE.DIRSET = (1<<4)+(1<<5)+(1<<6)+(1<<7); \
                             }   
                             
// The serial port(s) to be used for console
#define USE_USARTE0
#define init_serial_pins()       { \
                                    PORTE.OUTSET = (1<<3); \
                                    PORTE.DIRSET = (1<<3); \
                                  }  
                             
// Configuration reset jumper, see photo reset_CrumbX1-NET.png
#define RST_JUMPER_init()           { }
#define RST_JUMPER_write(value)     writeBit(PORTA.OUT,4,value)
#define RST_JUMPER_read             readBit(PORTA.IN,3)
#define RST_JUMPER_exit()           PORTA.OUTSET = (1<<4)

// Only 2 ADC inputs are wired on the module (PA0, PA1)
#define ADC_CHANNELS                2
   
// Chip select line, plus use the red LED as activity indicator
#define nCSMMC(value)           { \
                                  if (value==0) { \
                                     PORTE.OUTCLR=(1<<1); \
                                     PORTA.OUTCLR=(1<<4); \
                                  } else { \
                                     PORTE.OUTSET=(1<<1); \
                                     PORTA.OUTSET=(1<<4); \
                                  } \
                                }

// Port that is used for the SPI interface
#define MMC_PORT                PORTD
#define MMC_SPI                 SPID

// set nCSMMC, SS, MOSI and SCK to output mode. MISO is an input. The red LED is output.
#define init_mmc_pins()         { \
                                  PORTD.DIRSET=(1<<4)+(1<<5)+(1<<7); \
                                  PORTE.DIRSET=(1<<1); \
                                  PORTE.OUTSET=(1<<1); \
                                  PORTA.DIRSET=(1<<4); \
                                }

// Settings for the Email application
#define email_PORT       PORTC.IN

// Definition of i/o port bits that are protected.
// These pins cannot be accessed via io-commands d, p and o.
// 0x00 means that the whole port is not protected.
#define protect_PORTA        0x40
#define protect_PORTB        0x00
#define protect_PORTC        0x00
#ifdef CARDSLOT
    #define protect_PORTD    0xFE     
#else
    #define protect_PORTD    0x1E 
#endif
#define protect_PORTE        0xFF
#define protect_PORTF        0xFF    

// Number of shift register inputs and outputs
#define XL_INPUTS            24
#define XL_OUTPUTS           24

// Macros for additional inputs through shift registers (Port X)
// Some commands are repeated several times to slow down the serial interface a little
#define SHIFTin              { \
                               PORTD.OUTSET=(1<<4); \
                               PORTD.OUTSET=(1<<4); \
                               PORTD.OUTSET=(1<<4); \
                               PORTD.OUTCLR=(1<<4); \
                               PORTD.OUTCLR=(1<<4); \
                               PORTD.OUTCLR=(1<<4); \
                             }
#define DATAin               readBit(PORTD.IN,0)
#define LOADin               { \
                               PORTD.OUTCLR=(1<<0); \
                               PORTD.OUTCLR=(1<<0); \
                               PORTD.OUTCLR=(1<<0); \
                               PORTD.OUTSET=(1<<0); \
                               PORTD.OUTSET=(1<<0); \
                               PORTD.OUTSET=(1<<0); \
                               PORTD.OUTSET=(1<<0); \
                             } 

// Macros for additional outputs through shift registers (Port Y)
// Some commands are repeated several times to slow down the serial interface a little
#define STOREout             { \
                               PORTD.OUTSET=(1<<3); \
                               PORTD.OUTSET=(1<<3); \
                               PORTD.OUTSET=(1<<3); \
                               PORTD.OUTCLR=(1<<3); \
                               PORTD.OUTCLR=(1<<3); \
                               PORTD.OUTCLR=(1<<3); \
                             }
#define SHIFTout             { \
                               PORTD.OUTSET=(1<<4); \
                               PORTD.OUTSET=(1<<4); \
                               PORTD.OUTSET=(1<<4); \
                               PORTD.OUTCLR=(1<<4); \
                               PORTD.OUTCLR=(1<<4); \
                               PORTD.OUTCLR=(1<<4); \
                             }
#define DATAout(value)       writeBit(PORTD.OUT,2,value)

#endif // _HW_LAYOUT_CRUMBX1_NET_XL_H__

