/*

The inetd is called by uIP for each incoming TCP connection.
It calls the correct application depending on port number.

Port 68/udp -> dhdcp
Port 80/tcp -> httpd
Port 23/tcp -> socketd
Port 25/tcp -> smtpc

*/

#include "../uip/uip.h"
#include "inetd.h"
#include "../socketd/socketd.h"
#include "../httpd/httpd.h"
#include "../dhcpc/dhcpc.h"
#include "../smtpc/smtpc.h"

// Dispatch events for TCP connections to the correct application.
// This is called by uIP in case of events related to a connection 
// and also periodically.

void inetd_appcall(void) {
    // Get the application state structure for the current connection
    struct application_state *app_state = (struct application_state *) &(uip_conn->appstate);
    
    if (uip_conn->lport == HTONS(80)) {
        httpd_appcall(app_state);
    }    
    else if (uip_conn->lport == HTONS(23)) {
        socketd_appcall(app_state);
    }    
    #ifdef EMAIL_APP
        else if (uip_conn->rport == HTONS(25)) {
            smtpc_appcall(app_state);
        }
    #endif // EMAIL_APP
}


#if UIP_CONF_UDP
    // Dispatch events for UDP to the correct application.
    // This is called by uIP when an UDP packet has been received
    // and also periodically.
    void udp_appcall(void) {
        // Get the application state structure for the current connection
        struct udp_application_state *app_state = (struct udp_application_state *) &(uip_conn->appstate);
        
        if (uip_udp_conn->lport == HTONS(68)) {
            dhcpc_appcall(app_state);
        }
    }
#endif // UIP_CONF_UDP


// Initialize the applications and tell uIP on what ports we want to listen.

void inetd_init(void) {   
    #if UIP_CONF_UDP    
        dhcpc_init();
    #endif //UIP_CONF_UDP
    uip_listen(HTONS(80));
    uip_listen(HTONS(23));
}
