/*
Serial port daemon to call user_functions and io-commands.
 */

#include "seriald.h"
#include "../io-commands.h"
#include "../driver/serialconsole.h"
#include "../dhcpc/dhcpc.h"
#include "../uip/uip.h"
#include "../driver/CP2200.h"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

// Buffer for commands and results
char buffer[SERIAL_INPUT_BUFFER_SIZE];
    
void seriald_init(void) {
    initserial();
}

// Called from the main loop as often as possible
void seriald_main_loop(void) {
    if (receivedLine()) {
        fgets(buffer,sizeof(buffer),stdin);
        // remove the training line break
        int bytes=strlen(buffer);
        buffer[--bytes]=0;
        
        // we now use the buffer for both input and output to reduce RAM usage
        
        // output the current ip address if the command is "ip"
        if (strcmp(buffer,"ip")==0) {
            uint8_t adr[4];
            uip_gethostaddr(adr);  
            int len=sprintf_P(buffer,PSTR("ip=%u.%u.%u.%u\n"),adr[0],adr[1],adr[2],adr[3]); 
            // append the DHCP status, if DHCP is enabled
            #if UIP_CONF_UDP
                if (config.dhcpenable) {       
                    len+=get_dhcp_status(buffer+len);
                    buffer[len++]='\n';
                    buffer[len++]=0;
                }
            #endif // UIP_CONF_UDP
        }
        
        // execute the command using io-commands.c
        else {            
            io_command(buffer,buffer);
        }
        
        // print the result
        printf(buffer);
    }
}
